package com.fhkj.oltinspection.controller;

import javax.websocket.server.ServerEndpoint;

import com.fhkj.oltinspection.util.GetHttpSessionConfigurator;

import org.springframework.stereotype.Component;
import org.springframework.context.ApplicationContext;

import javax.servlet.http.HttpSession;
// import javax.servlet.http.HttpSession;
import javax.websocket.*;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArraySet;

@ServerEndpoint(value = "/pushInfo", configurator = GetHttpSessionConfigurator.class)
@Component
public class PushInfoController {
    // 静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
    private static int onlineCount = 0; 
    // concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
    private static CopyOnWriteArraySet<PushInfoController> webSocketSet = new CopyOnWriteArraySet<PushInfoController>();
    // 与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;

    private static Map<String, Session> map = new HashMap<>();// 存放当前登陆用户的session

    public static SimpleDateFormat sdf = new SimpleDateFormat("yyyy年MM月dd日 HH时mm分ss秒");

    private static ApplicationContext applicationContext;

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session, EndpointConfig config) {
        this.session = session;
        HttpSession httpSession = (HttpSession) config.getUserProperties().get(HttpSession.class.getName());
        /*System.out.println("name is : "+ httpSession.getAttribute("name"));
        System.out.println("the session is : "+session.getId());
        System.out.println("session content is : "+session.getId());
        System.out.println("httpSession is : "+httpSession.getId()); */

        // 获取路径传过来的username
        String str = session.getQueryString();
        String sendUser = str.split("=")[1];
        map.put(sendUser, session);
        webSocketSet.add(this); // 加入set中
        addOnlineCount(); // 在线数加1
        System.out.println("有新连接加入！当前在线人数为" + getOnlineCount() + "连接人id是：" + sendUser);
        try {
            sendMessage("已接收" + sendUser + "的消息"); // 发送消息的方法
        } catch (IOException e) {
            System.out.println("IO异常");
        }
    }

    /**
     * @return the applicationContext
     */
    public ApplicationContext getApplicationContext() {
        return applicationContext;
    }

    /**
     * @param applicationContext the applicationContext to set
     */
    public static void setApplicationContext(ApplicationContext applicationContext) {
        PushInfoController.applicationContext = applicationContext;
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        webSocketSet.remove(this); // 从set中删除
        subOnlineCount(); // 在线数减1
        System.out.println("有一连接关闭！当前在线人数为" + getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     * 
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        try {
            System.out.println("来自客户端的消息:" + message);
            sendMessage("我收到了你的消息");
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } // 发送消息的方法
    }

    /**
     * 发生错误时调用
     */
    @OnError
    public void onError(Session session, Throwable error) {
        System.out.println("发生错误");
        error.printStackTrace();
    }

    public void sendMessage(String message) throws IOException {
        this.session.getAsyncRemote().sendText(message);
    }


    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        PushInfoController.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        PushInfoController.onlineCount--;
    }
}
