package com.fhkj.oltinspection.util;

import java.io.IOException;
import java.util.Arrays;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.fhkj.oltinspection.entity.FtpConfigEntity;
import com.fhkj.oltinspection.pojo.Result;

import org.apache.commons.net.ftp.FTPClient;

/**
 * @author 胡懿
 * @see 这个了用来操作ftp
 */
public class FTPUtil {
    /**
     * @param ftpConfigEntity
     * @return Result
     * @see 验证ftp参数是否合法
     */
    private Result validateFtpParameter (FtpConfigEntity ftpConfigEntity) {
        if (null == ftpConfigEntity.getHost() || "".equals(ftpConfigEntity.getHost().trim())) {
            return new Result ("IP为空", false);
        } else if (!validateHost(ftpConfigEntity.getHost())) {
            return new Result ("IP地址不合法", false);
        }
        if (null == ftpConfigEntity.getPort() || "0".equals(ftpConfigEntity.getPort().toString())) {
            return new Result ("port为空", false);
        }
        if (null == ftpConfigEntity.getUsername() || "".equals(ftpConfigEntity.getUsername().trim())) {
            return new Result ("username为空", false);
        }
        if (null == ftpConfigEntity.getPassword() || "".equals(ftpConfigEntity.getPassword().trim())) {
            return new Result ("password为空", false);
        }
        return new Result ("验证成功", true);
    }
    /**
     * @param ip
     * @return boolean
     * @see 正则验证ip地址是否合法
     */
    private boolean validateHost (String ip) {
        String pattern = "(2(5[0-5]{1}|[0-4]\\d{1})|[0-1]?\\d{1,2})(\\.(2(5[0-5]{1}|[0-4]\\d{1})|[0-1]?\\d{1,2})){3}";
        Pattern r = Pattern.compile(pattern);
        Matcher m = r.matcher(ip);
        return m.matches();
    }
    /**
     * @see 用来来登录ftp
     * @param com.fhkj.oltinspection.entity.FtpConfigEntity
     * @exception IOException
     * @return Result
     */
    public Result loginFtp(FtpConfigEntity ftpConfigEntity) {
        if (ftpConfigEntity == null) {
            return new Result("在数据库中未能获取到ftp的连接配置信息", false);
        } else {
            Result result = validateFtpParameter(ftpConfigEntity);
            if (!(boolean)result.getState()) {
                return result;
            }
        }
        String host = ftpConfigEntity.getHost();
        Integer port = ftpConfigEntity.getPort();
        String username = ftpConfigEntity.getUsername();
        String password = ftpConfigEntity.getPassword();
        FTPClient ftpClient = new FTPClient(); // 创建ftp连接对象
        try {
            ftpClient.connect (host, port); // 设置ip和端口
            ftpClient.login (username, password); // 设置用户名和密码
            String[] ftpFileNames = ftpClient.listNames();
            System.out.println(Arrays.toString(ftpFileNames));
            if (ftpClient.isConnected()) { // 判断ftp是否连接
                ftpClient.logout(); // 退出ftp登录
                ftpClient.disconnect(); // 关闭连接
            }
        } catch (IOException e) {
            return new Result("出现连接异常", PackageExceptionUtil.exceptionToString(e), false);
        }
        return new Result("连接成功", true);
    }
}